# Generated by Django 3.2.15 on 2022-08-19 15:17

import ckeditor_uploader.fields
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0009_auto_20220818_2156'),
    ]

    operations = [
        migrations.CreateModel(
            name='PaymentOption',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extended_data', models.JSONField(default=dict, editable=False)),
                ('short_name', models.CharField(max_length=255, unique=True, verbose_name='Kurzname')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('notice', ckeditor_uploader.fields.RichTextUploadingField(blank=True, help_text='Wird unter der Option angezeigt', verbose_name='Anmerkung')),
                ('email_notice', ckeditor_uploader.fields.RichTextUploadingField(blank=True, help_text='Wird in der Bestätigungs-E-Mail angezeigt', verbose_name='E-Mail-Bemerkung')),
                ('price', models.FloatField(validators=[django.core.validators.MinValueValidator(0.0)])),
            ],
            options={
                'verbose_name': 'Payment option',
                'verbose_name_plural': 'Payment options',
            },
        ),
        migrations.RenameModel("ProcessingOption", "ShippingOption"),
        migrations.RenameModel("ProcessingOptionPrice", "ShippingOptionPrice"),
    ]



# Generated by Django 3.2.16 on 2022-11-09 20:29

import ckeditor_uploader.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0011_auto_20220819_1742'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='order',
            options={'permissions': [('manage_orders', 'Can manage orders')], 'verbose_name': 'Order', 'verbose_name_plural': 'Orders'},
        ),
        migrations.AlterModelOptions(
            name='orderform',
            options={'permissions': [('manage_orders_of_form', 'Can manage orders of form')], 'verbose_name': 'Order form', 'verbose_name_plural': 'Order forms'},
        ),
        migrations.AddField(
            model_name='item',
            name='image',
            field=models.ImageField(blank=True, null=True, upload_to='order/items', verbose_name='Image'),
        ),
        migrations.AlterField(
            model_name='item',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='item',
            name='notice',
            field=ckeditor_uploader.fields.RichTextUploadingField(blank=True, help_text='Shown below the item', verbose_name='Notice'),
        ),
        migrations.AlterField(
            model_name='item',
            name='short_name',
            field=models.CharField(max_length=255, unique=True, verbose_name='Short name'),
        ),
        migrations.AlterField(
            model_name='order',
            name='email',
            field=models.EmailField(max_length=254, verbose_name='Email'),
        ),
        migrations.AlterField(
            model_name='order',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='order',
            name='notes',
            field=models.TextField(blank=True, verbose_name='Notes'),
        ),
        migrations.AlterField(
            model_name='order',
            name='place',
            field=models.CharField(blank=True, max_length=255, verbose_name='Place'),
        ),
        migrations.AlterField(
            model_name='order',
            name='sent',
            field=models.BooleanField(default=False, verbose_name='Sent'),
        ),
        migrations.AlterField(
            model_name='order',
            name='shipping_option',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='order.shippingoption', verbose_name='Selected shipping option'),
        ),
        migrations.AlterField(
            model_name='order',
            name='street',
            field=models.CharField(blank=True, max_length=255, verbose_name='Street'),
        ),
        migrations.AlterField(
            model_name='orderform',
            name='available_shipping_options',
            field=models.ManyToManyField(related_name='forms', to='order.ShippingOption', verbose_name='Available shipping options'),
        ),
        migrations.AlterField(
            model_name='orderform',
            name='help_text',
            field=ckeditor_uploader.fields.RichTextUploadingField(blank=True, help_text="Shown in the form's footer", verbose_name='Help text'),
        ),
        migrations.AlterField(
            model_name='orderform',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='orderitem',
            name='count',
            field=models.IntegerField(verbose_name='Count'),
        ),
        migrations.AlterField(
            model_name='orderitem',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='orderitem',
            name='order',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='items', to='order.order', verbose_name='Order'),
        ),
        migrations.AlterField(
            model_name='paymentoption',
            name='email_notice',
            field=ckeditor_uploader.fields.RichTextUploadingField(blank=True, help_text='Shown in the confirmation email', verbose_name='Email notice'),
        ),
        migrations.AlterField(
            model_name='paymentoption',
            name='notice',
            field=ckeditor_uploader.fields.RichTextUploadingField(blank=True, help_text='Shown below the option', verbose_name='Notice'),
        ),
        migrations.AlterField(
            model_name='paymentoption',
            name='short_name',
            field=models.CharField(max_length=255, unique=True, verbose_name='Short name'),
        ),
        migrations.AlterField(
            model_name='shippingoption',
            name='address_necessary',
            field=models.BooleanField(default=False, verbose_name='Address necessary'),
        ),
        migrations.AlterField(
            model_name='shippingoption',
            name='email_notice',
            field=ckeditor_uploader.fields.RichTextUploadingField(blank=True, help_text='Shown in the confirmation email', verbose_name='Email notice'),
        ),
        migrations.AlterField(
            model_name='shippingoption',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='shippingoption',
            name='notice',
            field=ckeditor_uploader.fields.RichTextUploadingField(blank=True, help_text='Shown below the option', verbose_name='Notice'),
        ),
        migrations.AlterField(
            model_name='shippingoption',
            name='short_name',
            field=models.CharField(max_length=255, unique=True, verbose_name='Short name'),
        ),
        migrations.AlterField(
            model_name='shippingoptionprice',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='shippingoptionprice',
            name='option',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='prices', to='order.shippingoption', verbose_name='Shipping option'),
        ),
    ]



# Generated by Django 3.2.18 on 2023-03-09 20:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0018_auto_20221231_1841'),
    ]

    operations = [
        migrations.AlterField(
            model_name='digitalproductshare',
            name='share_expiration',
            field=models.DateTimeField(verbose_name='Share expiration'),
        ),
        migrations.AlterField(
            model_name='order',
            name='digital_products_key',
            field=models.TextField(blank=True, verbose_name='Digital products key'),
        ),
    ]



from django.contrib import admin

from guardian.admin import GuardedModelAdminMixin

from .models import (
    DigitalProduct,
    Item,
    NextcloudProvider,
    Order,
    OrderForm,
    OrderItem,
    PaymentOption,
    ShippingOption,
    ShippingOptionPrice,
)


class ItemInline(admin.TabularInline):
    model = Item


class ItemAdmin(admin.ModelAdmin):
    model = Item


admin.site.register(Item, ItemAdmin)


class OrderFormAdmin(GuardedModelAdminMixin, admin.ModelAdmin):
    model = OrderForm


admin.site.register(OrderForm, OrderFormAdmin)


class OrderItemInline(admin.TabularInline):
    model = OrderItem


class OrderAdmin(admin.ModelAdmin):
    model = Order
    readonly_fields = ["confirm_key", "form"]
    list_display = ["__str__", "submitted", "confirmed", "paid", "sent"]
    inlines = [OrderItemInline]


admin.site.register(Order, OrderAdmin)


class ShippingOptionPriceInline(admin.TabularInline):
    model = ShippingOptionPrice


class ShippingOptionAdmin(admin.ModelAdmin):
    model = ShippingOption
    inlines = [ShippingOptionPriceInline]


admin.site.register(ShippingOption, ShippingOptionAdmin)


class PaymentOptionAdmin(admin.ModelAdmin):
    model = PaymentOption


admin.site.register(PaymentOption, PaymentOptionAdmin)

admin.site.register(NextcloudProvider)
admin.site.register(DigitalProduct)



from datetime import timedelta

from django.utils import timezone

from aleksis.core.celery import app

from .models import DigitalProductShare


@app.task(run_every=timedelta(hours=1))
def cleanup_expired_shares():
    """Delete expired digital product shares."""
    expired_shares = DigitalProductShare.objects.filter(share_expiration__lt=timezone.now())
    expired_shares.delete()



from django.db.models import Exists, OuterRef

import graphene
from graphene_django import DjangoObjectType

from aleksis.apps.order.models import (
    DigitalProduct,
    DigitalProductShare,
    Item,
    Order,
    OrderForm,
    OrderItem,
    PaymentOption,
    ShippingOption,
    ShippingOptionPrice,
)
from aleksis.core.util.core_helpers import has_person


class ItemType(DjangoObjectType):
    def resolve_image(root, info, **kwargs):
        if root.image:
            return info.context.build_absolute_uri(root.image.url)
        return ""

    class Meta:
        model = Item


class DigitalProductType(DjangoObjectType):
    class Meta:
        model = DigitalProduct


class DigitalProductShareType(DjangoObjectType):
    class Meta:
        model = DigitalProductShare


class ItemWithDigitalProductType(DjangoObjectType):
    def resolve_image(root, info, **kwargs):
        if root.image:
            return info.context.build_absolute_uri(root.image.url)
        return ""

    class Meta:
        model = Item


class OrderItemWithDigitalProductType(DjangoObjectType):
    item = graphene.Field(ItemWithDigitalProductType)
    shares_left = graphene.Int()
    shares = graphene.List(DigitalProductShareType)

    def resolve_shares(self, info, **kwargs):
        return self.current_shares

    class Meta:
        model = OrderItem


class ShippingOptionPriceType(DjangoObjectType):
    class Meta:
        model = ShippingOptionPrice


class ShippingOptionType(DjangoObjectType):
    class Meta:
        model = ShippingOption


class PaymentOptionType(DjangoObjectType):
    class Meta:
        model = PaymentOption


class OrderFormType(DjangoObjectType):
    class Meta:
        model = OrderForm


class OrderItemType(DjangoObjectType):
    class Meta:
        model = OrderItem


class OrderType(DjangoObjectType):
    class Meta:
        model = Order


class DigitalProductsOrderType(DjangoObjectType):
    class Meta:
        model = Order
        fields = ["id", "form", "digital_products_key"]


class OrderItemInputType(graphene.InputObjectType):
    item = graphene.ID(required=True)
    count = graphene.Int(required=True)


class OrderInputType(graphene.InputObjectType):
    full_name = graphene.String(required=True)
    email = graphene.String(required=True)
    notes = graphene.String(required=False)
    shipping_option = graphene.ID(required=True)
    payment_option = graphene.ID(required=True)
    items = graphene.List(OrderItemInputType, required=True)
    shipping_full_name = graphene.String(required=False)
    second_address_row = graphene.String(required=False)
    street = graphene.String(required=False)
    housenumber = graphene.String(required=False)
    plz = graphene.String(required=False)
    place = graphene.String(required=False)


class DigitalProductsType(graphene.ObjectType):
    order = graphene.Field(DigitalProductsOrderType)
    items = graphene.List(OrderItemWithDigitalProductType)

    def resolve_order(self, info, **kwargs):
        return self

    def resolve_items(self, info, **kwargs):
        return self.items.filter(
            item__digital_product_object__isnull=False, item__digital_product_object__published=True
        )


class SendOrderMutation(graphene.Mutation):
    class Arguments:
        access_code = graphene.String(required=False)
        order_form = graphene.ID(required=True)
        order = OrderInputType(required=True)

    ok = graphene.Boolean()

    def mutate(self, info, **kwargs):
        order_form = OrderForm.objects.get(id=int(kwargs["order_form"]))
        if not order_form.has_user_access(info.context, kwargs["access_code"]):
            raise Exception("Access denied")
        order_args = kwargs["order"]

        items = [
            (order_form.available_items.get(id=int(ordered_item["item"])), ordered_item["count"])
            for ordered_item in order_args["items"]
            if ordered_item["count"] > 0
        ]

        # Ensure that the correct shipping option is used in all cases
        items_with_normal_shipping = any(item for item in items if not item[0].digital_product)
        items_with_digital_product = any(item for item in items if item[0].digital_product)
        shipping_options = order_form.available_shipping_options.all()
        if items_with_normal_shipping:
            shipping_options = shipping_options.exclude(for_digital_products=True)
        if not items_with_normal_shipping and items_with_digital_product:
            shipping_options = shipping_options.exclude(for_digital_products=False)

        shipping_option = shipping_options.get(id=int(order_args["shipping_option"]))
        payment_option = order_form.available_payment_options.get(
            id=int(order_args["payment_option"])
        )

        order = Order.objects.create(
            form=order_form,
            full_name=order_args["full_name"],
            email=order_args["email"],
            notes=order_args["notes"],
            shipping_option=shipping_option,
            payment_option=payment_option,
        )

        if has_person(info.context):
            order.person = info.context.user.person
            order.save()

        if shipping_option.address_necessary:
            order.shipping_full_name = order_args["shipping_full_name"]
            order.second_address_row = order_args["second_address_row"]
            order.street = order_args["street"]
            order.housenumber = order_args["housenumber"]
            order.plz = order_args["plz"]
            order.place = order_args["place"]
            order.save()

        for ordered_item, count in items:
            OrderItem.objects.create(item=ordered_item, count=count, order=order)

        order.submit(info.context)

        return SendOrderMutation(ok=True)


class CreateShareMutation(graphene.Mutation):
    class Arguments:
        key = graphene.String(required=True)
        order_item = graphene.ID(required=True)

    ok = graphene.Boolean()
    share = graphene.Field(DigitalProductShareType)

    def mutate(self, info, key, order_item, **kwargs):
        order_item = OrderItem.objects.filter(
            item__digital_product_object__isnull=False,
            item__digital_product_object__published=True,
            id=order_item,
        ).first()
        if not order_item:
            raise Exception("Invalid order item")
        if order_item.order.digital_products_key != key and not (
            has_person(info.context) and order_item.order.person == info.context.user.person
        ):
            raise Exception("Access denied")

        if order_item.shares_left > 0:
            share = order_item.create_share()

        return CreateShareMutation(ok=True, share=share)


class Query(graphene.ObjectType):
    order_form_by_id = graphene.Field(
        OrderFormType, id=graphene.ID(), access_code=graphene.String(required=False)
    )
    digital_products_by_key = graphene.Field(
        DigitalProductsType, key=graphene.String(required=True)
    )
    digital_products_by_person = graphene.List(DigitalProductsType)

    def resolve_order_form_by_id(self, info, id, access_code=""):  # noqa
        order_form = OrderForm.objects.get(id=int(id))
        if order_form.has_user_access(info.context, access_code):
            return order_form
        return None

    def resolve_digital_products_by_key(self, info, key):
        order = Order.objects.filter(paid=True).get(digital_products_key=key)
        return order

    def resolve_digital_products_by_person(self, info, **kwargs):
        if not has_person(info.context):
            return []
        orders = Order.objects.annotate(
            digital_products_exist=Exists(
                OrderItem.objects.filter(
                    order=OuterRef("pk"),
                    item__digital_product_object__isnull=False,
                    item__digital_product_object__published=True,
                )
            )
        ).filter(digital_products_exist=True, paid=True, person=info.context.user.person)
        return orders


class Mutation(graphene.ObjectType):
    send_order = SendOrderMutation.Field()
    create_share = CreateShareMutation.Field()



# Generated by Django 3.2.15 on 2022-08-18 19:56

import ckeditor_uploader.fields
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0008_auto_20210121_1154'),
    ]

    operations = [
        migrations.AddField(
            model_name='item',
            name='count_for_shipping',
            field=models.BooleanField(default=True, verbose_name='Count for shipping'),
        ),
    ]



import rules

from aleksis.core.util.predicates import (
    has_any_object,
    has_global_perm,
    has_object_perm,
    has_person,
)

from .models import OrderForm, OrderItem
from .util.predicates import has_order_perm


@rules.predicate
def has_any_digital_product(user, obj):
    return OrderItem.objects.filter(
        order__person=user.person,
        order__paid=True,
        item__digital_product_object__isnull=False,
        item__digital_product_object__published=True,
    ).exists()


manage_orders_of_form_predicate = has_person & (
    has_global_perm("order.manage_orders") | has_object_perm("order.manage_orders_of_form")
)
rules.add_perm("order.manage_orders_of_form_rule", manage_orders_of_form_predicate)

view_orders_predicate = has_person & (
    has_global_perm("order.manage.orders")
    | has_any_object("order.manage_orders_of_form", OrderForm)
)
rules.add_perm("order.view_orders_rule", view_orders_predicate)

manage_order_predicate = has_person & (has_global_perm("order.manage_orders") | has_order_perm)
rules.add_perm("order.manage_order_rule", manage_order_predicate)

view_my_digital_products_predicate = has_person & (has_any_digital_product)
rules.add_perm("order.view_my_digital_products_rule", view_my_digital_products_predicate)

view_menu_predicate = view_orders_predicate | view_my_digital_products_predicate
rules.add_perm("order.view_menu_rule", view_menu_predicate)



import json

import pytest
from graphene_django.utils.testing import graphql_query

from aleksis.apps.order.models import OrderForm


@pytest.fixture
def graphql_user(django_user_model):
    username = "foo"
    password = "bar"

    if not django_user_model.objects.filter(username=username).exists():
        django_user_model.objects.create_user(username=username, password=password)
    return django_user_model.objects.get(username=username)


@pytest.fixture
def client_query(client, graphql_user):
    client.login(username=graphql_user.username, password="bar")

    def func(*args, **kwargs):
        return graphql_query(*args, **kwargs, client=client)

    return func


@pytest.fixture
def order_form():
    return OrderForm.objects.create(
        title="Test Order Form",
        access_code="secret_code",
        from_name="Test From",
        from_email="test@example.com",
    )


@pytest.fixture
def order_form_public(order_form):
    order_form.access_code = ""
    order_form.save()
    return order_form


def test_order_form_by_id_secured_without_login_without_access_code(client_query, order_form):
    response = client_query(
        """
        query($id: Int!) {
            orderFormById(id: $id) {
                title
            }
        }
        """,
        variables={"id": order_form.id},
    )
    print(response.content)
    content = json.loads(response.content)
    print(content)
    assert content["data"]["orderFormById"]["name"] == "Test Order Form"


def test_order_form_by_id_secured_without_login_with_access_code(client_query, order_form):
    response = client_query(
        """
        query($id: Int!) {
            orderFormById(id: $id, accessCode: "secret_code") {
                title
            }
        }
        """,
        variables={"id": order_form.id},
    )
    print(response.content)
    content = json.loads(response.content)
    print(content)
    assert not content["data"]["orderFormById"]


# def test_order_form_by_id_secured_with_login_without_access_code(client_query, order_form):



# Generated by Django 3.2.15 on 2022-08-19 15:42

import ckeditor_uploader.fields
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0010_auto_20220819_1717'),
    ]

    operations = [
        migrations.RenameField(
            model_name='order',
            old_name='processing_option',
            new_name='shipping_option',
        ),
        migrations.RenameField(
            model_name='orderform',
            old_name='available_processing_options',
            new_name='available_shipping_options',
        ),
        migrations.AddField(
            model_name='order',
            name='payment_option',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='order.paymentoption', verbose_name='Selected payment option'),
        ),
        migrations.AddField(
            model_name='orderform',
            name='available_payment_options',
            field=models.ManyToManyField(related_name='forms', to='order.PaymentOption', verbose_name='Available payment options'),
        ),
    ]



from tempfile import TemporaryDirectory
from typing import Any

from django.contrib import messages
from django.core.exceptions import SuspiciousOperation
from django.http import HttpRequest, HttpResponse
from django.http.response import FileResponse, HttpResponseNotFound
from django.shortcuts import get_object_or_404, render
from django.urls import reverse_lazy
from django.utils.translation import gettext as _
from django.views.generic import FormView
from django.views.generic.base import View
from django.views.generic.detail import DetailView

from django_filters.views import FilterView
from django_tables2 import SingleTableMixin
from rules.contrib.views import PermissionRequiredMixin

from aleksis.apps.order.filters import OrderFilter
from aleksis.apps.order.pdf import generate_address_labels, generate_order_labels
from aleksis.apps.order.tables import OrderTable
from aleksis.core.mixins import AdvancedDeleteView, AdvancedEditView
from aleksis.core.util.core_helpers import queryset_rules_filter
from aleksis.core.util.pdf import render_pdf

from .forms import OrderActionForm, PickUpForm
from .models import Order, OrderForm


class OrderConfirmView(View):
    def get(self, request: HttpRequest, key, **kwargs: Any) -> HttpResponse:
        try:
            self.order = Order.objects.get(confirm_key=key)
        except Order.DoesNotExist:
            return HttpResponseNotFound()
        if not self.order.confirmed:
            self.order.confirm(request)
        return render(request, "order/confirmed.html", context={"order": self.order})


class OrderDetailView(PermissionRequiredMixin, DetailView):
    model = Order
    permission_required = "order.manage_order_rule"
    template_name = "order/detail.html"


class OrderEditView(PermissionRequiredMixin, AdvancedEditView):
    model = Order
    fields = (
        "full_name",
        "email",
        "notes",
        "submitted",
        "confirmed",
        "paid",
        "sent",
        "shipping_option",
        "shipping_full_name",
        "second_address_row",
        "street",
        "housenumber",
        "plz",
        "place",
        "payment_option",
    )
    permission_required = "order.manage_order_rule"
    template_name = "order/edit.html"
    success_message = _("The order has been changed successfully.")
    success_url = reverse_lazy("list_orders")

    def post(self, request, *args, **kwargs):
        r = super().post(request, *args, **kwargs)

        order = self.get_object()

        if request.POST.get("mark-as-paid"):
            order.paid = True
            order.save()

        elif request.POST.get("mark-as-paid-email"):
            order.send_pay_confirmation()

        return r


class OrderDeleteView(PermissionRequiredMixin, AdvancedDeleteView):
    model = Order
    permission_required = "order.manage_order_rule"
    template_name = "core/pages/delete.html"
    success_message = _("The order has been deleted successfully.")
    success_url = reverse_lazy("list_orders")


class OrderListView(PermissionRequiredMixin, SingleTableMixin, FilterView):
    table_class = OrderTable
    model = Order
    template_name = "order/list.html"
    permission_required = "order.view_orders_rule"
    table_pagination = {"per_page": 50}

    filterset_class = OrderFilter

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["table"].request = self.request
        context["forms"] = queryset_rules_filter(
            self.request.user, OrderForm.objects.all(), "order.manage_orders_of_form_rule"
        )
        self.action_form = OrderActionForm(
            self.request, self.request.POST or None, queryset=self.get_queryset()
        )
        context["action_form"] = self.action_form
        return context

    def post(self, request, *args, **kwargs):
        r = self.get(request, *args, **kwargs)

        if "action" in self.request.POST and self.action_form.is_valid():
            self.action_form.execute()

        if request.POST.get("all-action"):
            if request.POST.get("print"):
                return render_pdf(request, "order/list_print.html", self.get_context_data())
            elif request.POST.get("packing-lists"):
                return render_pdf(request, "order/packing_lists.html", self.get_context_data())

        # Action for a single object
        if request.POST.get("single-action-pk"):
            order = get_object_or_404(Order, pk=request.POST.get("single-action-pk"))

            if request.POST.get("mark-as-paid"):
                order.paid = True
                order.save()

            elif request.POST.get("mark-as-paid-email"):
                order.send_pay_confirmation()

            elif request.POST.get("mark-as-sent"):
                order.sent = True
                order.save()

        # Actions for all objects
        elif request.POST.get("all-action"):
            if request.POST.get("mark-as-paid"):
                self.object_list.update(paid=True)

            elif request.POST.get("mark-as-paid-email"):
                self.object_list.update(paid=True)
                for order in self.object_list:
                    order.send_pay_confirmation()
            elif request.POST.get("mark-as-sent"):
                self.object_list.update(sent=True)

            elif request.POST.get("labels"):
                with TemporaryDirectory() as temp_dir:
                    filename = generate_order_labels(self.object_list, temp_dir)
                    with open(filename, "rb") as f:
                        return FileResponse(f, filename="labels.pdf")
            elif request.POST.get("address-labels"):
                with TemporaryDirectory() as temp_dir:
                    filename = generate_address_labels(self.object_list, temp_dir)
                    with open(filename, "rb") as f:
                        return FileResponse(f, filename="labels.pdf")
        return r


class OrderPickUpFormView(PermissionRequiredMixin, FormView):
    template_name = "order/pickup.html"
    permission_required = "order.view_orders_rule"
    form_class = PickUpForm

    def post(self, request, *args, **kwargs):
        if request.POST.get("order_id_action"):
            try:
                order_id = int(request.POST["order_id_action"])
            except ValueError as e:
                raise SuspiciousOperation() from e
            try:
                order = Order.objects.get(pk=order_id)
            except Order.DoesNotExist as e:
                raise SuspiciousOperation() from e

            if request.POST.get("abort"):
                return self.get(request, *args, **kwargs)
            elif request.POST.get("collected") and order.paid and not order.sent:
                order.sent = True
                order.save()
                messages.success(request, _("The order was successfully marked as collected."))
            elif request.POST.get("manually-paid-collected") and not order.paid and not order.sent:
                order.sent = True
                order.paid = True
                order.notes += _("\nManually paid")
                order.save()
                messages.success(
                    request, _("The order was successfully marked as manually paid and collected.")
                )

            elif request.POST.get("paid-collected") and not order.paid and not order.sent:
                order.sent = True
                order.paid = True
                order.save()
                messages.success(
                    request, _("The order was successfully marked as normally paid and collected.")
                )

        return super().post(request, *args, **kwargs)

    def form_valid(self, form) -> HttpResponse:
        context = self.get_context_data()
        order = form.cleaned_data["order_id"]
        print(order)
        context["order"] = order
        context["form"] = PickUpForm()

        return self.render_to_response(context)



# Generated by Django 3.2.16 on 2022-12-22 12:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0014_order_person'),
    ]

    operations = [
        migrations.AddField(
            model_name='item',
            name='order',
            field=models.IntegerField(default=0, verbose_name='Order'),
        ),
    ]



from django import forms
from django.core.exceptions import ValidationError
from django.db.models import Exists, OuterRef
from django.utils.translation import gettext as _

from aleksis.core.forms import ActionForm

from .models import Order, OrderItem


def send_confirmation_reminder(modeladmin, request, queryset):
    qs = queryset.filter(submitted=True, confirmed=False)
    for order in qs:
        order.send_confirmation_reminder(request)


send_confirmation_reminder.short_description = _("Send confirmation reminder")


def send_pay_reminder(modeladmin, request, queryset):
    qs = queryset.filter(submitted=True, confirmed=True, paid=False)
    for order in qs:
        order.send_pay_reminder(request)


send_pay_reminder.short_description = _("Send pay reminder")


def send_digital_products_link(modeladmin, request, queryset):
    qs = queryset.annotate(
        digital_products_exist=Exists(
            OrderItem.objects.filter(
                order=OuterRef("pk"),
                item__digital_product_object__isnull=False,
                item__digital_product_object__published=True,
            )
        )
    ).filter(digital_products_exist=True, submitted=True, confirmed=True, paid=True)
    for order in qs:
        order.send_digital_products_link(request)


send_digital_products_link.short_description = _("Send digital products link")


class OrderActionForm(ActionForm):
    actions = [send_confirmation_reminder, send_pay_reminder, send_digital_products_link]


class PickUpForm(forms.Form):
    order_id = forms.CharField(
        label=_("Order id"), widget=forms.TextInput(attrs={"autofocus": "autofocus"})
    )

    def clean_order_id(self):
        order_id = self.cleaned_data["order_id"]
        order_id = order_id.strip()
        try:
            order_id = int(order_id)
        except ValueError:
            raise ValidationError(_("You must enter a valid number as order id.")) from None
        try:
            order = Order.objects.get(pk=order_id)
        except Order.DoesNotExist:
            raise ValidationError(_("This order doesn't exist.")) from None
        return order



# Generated by Django 3.2.16 on 2022-12-14 19:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0012_auto_20221109_2129'),
    ]

    operations = [
        migrations.AddField(
            model_name='item',
            name='digital_product',
            field=models.BooleanField(default=False, verbose_name='Digital product'),
        ),
        migrations.AddField(
            model_name='shippingoption',
            name='for_digital_products',
            field=models.BooleanField(default=False, verbose_name='For digital products'),
        ),
    ]



# Generated by Django 3.2.16 on 2022-12-31 13:22

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0016_auto_20221231_1319'),
    ]

    operations = [
        migrations.CreateModel(
            name='DigitalProductShare',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extended_data', models.JSONField(default=dict, editable=False)),
                ('share_id', models.CharField(max_length=255, verbose_name='Share ID')),
                ('share_url', models.URLField(verbose_name='Share URL')),
                ('share_expiration', models.DateField(verbose_name='Share expiration')),
                ('order_item', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='order.orderitem', verbose_name='Order item')),
            ],
            options={
                'verbose_name': 'Digital product share',
                'verbose_name_plural': 'Digital product shares',
            },
        ),
    ]



# Generated by Django 3.2.16 on 2022-12-31 12:19

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0015_item_order'),
    ]

    operations = [
        migrations.CreateModel(
            name='NextcloudProvider',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extended_data', models.JSONField(default=dict, editable=False)),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('url', models.URLField(verbose_name='URL')),
                ('username', models.CharField(max_length=255, verbose_name='Username')),
                ('password', models.CharField(max_length=255, verbose_name='Password')),
            ],
            options={
                'verbose_name': 'Nextcloud provider',
                'verbose_name_plural': 'Nextcloud providers',
            },
        ),
        migrations.CreateModel(
            name='DigitalProduct',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extended_data', models.JSONField(default=dict, editable=False)),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('nextcloud_path', models.TextField(verbose_name='Nextcloud path')),
                ('published', models.BooleanField(default=False, verbose_name='Published')),
                ('nextcloud_provider', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='order.nextcloudprovider', verbose_name='Nextcloud provider')),
            ],
            options={
                'verbose_name': 'Digital product',
                'verbose_name_plural': 'Digital products',
            },
        ),
        migrations.AddField(
            model_name='item',
            name='digital_product_object',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='order.digitalproduct', verbose_name='Digital product'),
        ),
    ]



from datetime import datetime, timedelta
from urllib.parse import urljoin
from uuid import uuid4

from django.core.validators import MinValueValidator
from django.db import models
from django.db.models import Q
from django.urls import reverse
from django.utils import timezone
from django.utils.translation import gettext as _

import requests
from ckeditor_uploader.fields import RichTextUploadingField
from defusedxml import ElementTree
from model_utils.models import TimeStampedModel
from templated_email import send_templated_mail

from aleksis.core.mixins import ExtensibleModel
from aleksis.core.util.core_helpers import has_person


class NextcloudProvider(ExtensibleModel):
    """Nextcloud provider for file storage."""

    name = models.CharField(max_length=255, verbose_name=_("Name"))
    url = models.URLField(verbose_name=_("URL"))
    username = models.CharField(max_length=255, verbose_name=_("Username"))
    password = models.CharField(max_length=255, verbose_name=_("Password"))

    def __str__(self):
        return self.name

    def build_url(self, path):
        """Build URL for given path."""
        return urljoin(self.url, path)

    def do_request(self, path, method="GET", data=None):
        """Build request for given path."""
        url = self.build_url(path)
        headers = {"OCS-APIRequest": "true"}
        r = requests.request(
            method, url, data=data, auth=(self.username, self.password), headers=headers
        )
        if not r.ok:
            raise Exception(r.text)
        xml = ElementTree.fromstring(r.content)
        return xml

    def test_connection(self):
        """Test connection to Nextcloud provider."""
        r = self.do_request("/ocs/v1.php/cloud/capabilities")
        return r.find("meta").find("status").text == "ok"

    class Meta:
        verbose_name = _("Nextcloud provider")
        verbose_name_plural = _("Nextcloud providers")


class DigitalProductShare(ExtensibleModel):
    """Digital product share."""

    order_item = models.ForeignKey(
        "OrderItem", on_delete=models.CASCADE, verbose_name=_("Order item"), related_name="shares"
    )
    share_id = models.CharField(max_length=255, verbose_name=_("Share ID"))
    share_url = models.URLField(verbose_name=_("Share URL"))
    share_expiration = models.DateTimeField(verbose_name=_("Share expiration"))

    def __str__(self):
        return self.share_id

    class Meta:
        verbose_name = _("Digital product share")
        verbose_name_plural = _("Digital product shares")


class DigitalProduct(ExtensibleModel):
    name = models.CharField(max_length=255, verbose_name=_("Name"))
    nextcloud_provider = models.ForeignKey(
        NextcloudProvider, on_delete=models.CASCADE, verbose_name=_("Nextcloud provider")
    )
    nextcloud_path = models.TextField(verbose_name=_("Nextcloud path"))
    published = models.BooleanField(default=False, verbose_name=_("Published"))

    def get_args_for_share(self, order, expire_date=None):
        if not expire_date:
            expire_date = (timezone.now() + timedelta(days=1)).date()
        return {
            "path": self.nextcloud_path,
            "shareType": 3,
            "permissions": 1,
            "expireDate": expire_date.strftime("%Y-%m-%d"),
            "label": _("Personal download link for {name}").format(name=order.full_name),
        }

    def create_share(self, order_item):
        args = self.get_args_for_share(order_item.order)
        xml = self.nextcloud_provider.do_request(
            "/ocs/v1.php/apps/files_sharing/api/v1/shares", method="POST", data=args
        )
        share_id = xml.find("data").find("id").text
        share_url = xml.find("data").find("url").text
        share_expiration = datetime.strptime(
            xml.find("data").find("expiration").text, "%Y-%m-%d %H:%M:%S"
        )

        return DigitalProductShare.objects.create(
            order_item=order_item,
            share_id=share_id,
            share_url=share_url,
            share_expiration=share_expiration,
        )

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _("Digital product")
        verbose_name_plural = _("Digital products")


class Item(ExtensibleModel):
    short_name = models.CharField(max_length=255, verbose_name=_("Short name"), unique=True)
    name = models.CharField(max_length=255, verbose_name=_("Name"))
    notice = RichTextUploadingField(
        verbose_name=_("Notice"), help_text=_("Shown below the item"), blank=True
    )
    price = models.FloatField(validators=[MinValueValidator(0.0)])
    max_count = models.IntegerField(
        default=0, validators=[MinValueValidator(0)], verbose_name=_("Maximum count of items")
    )
    count_for_shipping = models.BooleanField(default=True, verbose_name=_("Count for shipping"))
    image = models.ImageField(
        upload_to="order/items", blank=True, null=True, verbose_name=_("Image")
    )
    digital_product = models.BooleanField(default=False, verbose_name=_("Digital product"))
    digital_product_object = models.ForeignKey(
        DigitalProduct,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        verbose_name=_("Digital product"),
    )
    order = models.IntegerField(default=0, verbose_name=_("Order"))

    def __str__(self):
        return self.short_name

    class Meta:
        verbose_name = _("Item")
        verbose_name_plural = _("Items")


class PaymentOption(ExtensibleModel):
    short_name = models.CharField(max_length=255, verbose_name=_("Short name"), unique=True)
    name = models.CharField(max_length=255, verbose_name=_("Name"))
    notice = RichTextUploadingField(
        verbose_name=_("Notice"), help_text=_("Shown below the option"), blank=True
    )
    email_notice = RichTextUploadingField(
        verbose_name=_("Email notice"), help_text=_("Shown in the confirmation email"), blank=True
    )
    price = models.FloatField(validators=[MinValueValidator(0.0)])

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _("Payment option")
        verbose_name_plural = _("Payment options")


class ShippingOption(ExtensibleModel):
    short_name = models.CharField(max_length=255, verbose_name=_("Short name"), unique=True)
    name = models.CharField(max_length=255, verbose_name=_("Name"))
    notice = RichTextUploadingField(
        verbose_name=_("Notice"), help_text=_("Shown below the option"), blank=True
    )
    email_notice = RichTextUploadingField(
        verbose_name=_("Email notice"), help_text=_("Shown in the confirmation email"), blank=True
    )
    address_necessary = models.BooleanField(default=False, verbose_name=_("Address necessary"))
    for_digital_products = models.BooleanField(
        default=False, verbose_name=_("For digital products")
    )

    def get_price(self, count):
        for price in self.prices.order_by("price"):
            if price.min_count <= count <= price.max_count:
                return price.price
        return 0

    def __str__(self):
        return self.short_name


class ShippingOptionPrice(ExtensibleModel):
    option = models.ForeignKey(
        ShippingOption, models.CASCADE, related_name="prices", verbose_name=_("Shipping option")
    )
    min_count = models.IntegerField(
        verbose_name=_("Min count of items"), validators=[MinValueValidator(0)]
    )
    max_count = models.IntegerField(
        verbose_name=_("Max count of items"), validators=[MinValueValidator(0)]
    )
    price = models.FloatField(validators=[MinValueValidator(0.0)])

    def __str__(self):
        return f"{self.option}: {self.min_count} <= x <= {self.max_count}"


class OrderForm(ExtensibleModel):
    title = models.CharField(max_length=255, verbose_name=_("Form title"))
    available_items = models.ManyToManyField(
        to=Item, related_name="forms", verbose_name=_("Available items")
    )
    available_shipping_options = models.ManyToManyField(
        to=ShippingOption, related_name="forms", verbose_name=_("Available shipping options")
    )
    available_payment_options = models.ManyToManyField(
        to=PaymentOption, related_name="forms", verbose_name=_("Available payment options")
    )
    help_text = RichTextUploadingField(
        verbose_name=_("Help text"), help_text=_("Shown in the form's footer"), blank=True
    )
    from_email = models.EmailField(verbose_name=_("From email address"))
    from_name = models.CharField(verbose_name=_("From name"), max_length=255)
    access_code = models.CharField(max_length=2555, blank=True, verbose_name=_("Access code"))
    closed = models.BooleanField(default=False, verbose_name=_("Form closed for orders"))
    sender = models.TextField(blank=True, verbose_name=_("Sender"))

    def __str__(self):
        return self.title

    @property
    def email_sender(self):
        return f"{self.from_name} <{self.from_email}>"

    @property
    def total(self):
        return sum([order.total for order in self.orders.all()])

    @property
    def items_count(self):
        return sum([order.items_count for order in self.orders.all()])

    @property
    def confirmed_count(self):
        return sum([order.items_count for order in self.orders.all() if order.confirmed])

    @property
    def paid_count(self):
        return sum([order.items_count for order in self.orders.all() if order.paid])

    @property
    def sent_count(self):
        return sum([order.items_count for order in self.orders.all() if order.sent])

    @property
    def annotated_items(self):
        item_list = []
        for item in self.available_items.all():
            item.items_count = sum(
                [
                    order_item.count
                    for order_item in OrderItem.objects.filter(order__form=self, item=item)
                ]
            )
            item.confirmed_count = sum(
                [
                    order_item.count
                    for order_item in OrderItem.objects.filter(
                        order__form=self, item=item, order__confirmed=True
                    )
                ]
            )
            item.paid_count = sum(
                [
                    order_item.count
                    for order_item in OrderItem.objects.filter(
                        order__form=self, item=item, order__paid=True
                    )
                ]
            )
            item.sent_count = sum(
                [
                    order_item.count
                    for order_item in OrderItem.objects.filter(
                        order__form=self, item=item, order__sent=True
                    )
                ]
            )
            item.items_total = sum(
                [
                    order_item.total
                    for order_item in OrderItem.objects.filter(order__form=self, item=item)
                ]
            )
            item_list.append(item)
        return item_list

    def has_user_access(self, request, access_code):
        access_granted = (
            has_person(request) or self.access_code.strip() == access_code.strip()
        ) and not self.closed
        return access_granted

    class Meta:
        verbose_name = _("Order form")
        verbose_name_plural = _("Order forms")
        permissions = [("manage_orders_of_form", _("Can manage orders of form"))]


class Order(ExtensibleModel, TimeStampedModel):
    form = models.ForeignKey(
        to=OrderForm, on_delete=models.CASCADE, verbose_name=_("Order form"), related_name="orders"
    )

    person = models.ForeignKey(
        to="core.Person", on_delete=models.SET_NULL, null=True, blank=True, verbose_name=_("Person")
    )

    full_name = models.CharField(max_length=255, verbose_name=_("First and last name"))
    email = models.EmailField(verbose_name=_("Email"))
    notes = models.TextField(verbose_name=_("Notes"), blank=True)

    submitted = models.BooleanField(default=False, verbose_name=_("Submitted"))
    confirmed = models.BooleanField(default=False, verbose_name=_("Confirmed"))
    confirm_key = models.TextField(verbose_name=_("Confirm key"), blank=True)
    paid = models.BooleanField(default=False, verbose_name=_("Paid"))
    sent = models.BooleanField(default=False, verbose_name=_("Sent"))

    shipping_option = models.ForeignKey(
        ShippingOption,
        models.SET_NULL,
        null=True,
        blank=True,
        verbose_name=_("Selected shipping option"),
    )
    shipping_full_name = models.CharField(
        verbose_name=_("First and last name"), max_length=255, blank=True
    )
    second_address_row = models.CharField(
        verbose_name=_("Second address row"), max_length=255, blank=True
    )
    street = models.CharField(verbose_name=_("Street"), max_length=255, blank=True)
    housenumber = models.CharField(verbose_name=_("Housenumber"), max_length=255, blank=True)
    plz = models.CharField(verbose_name=_("PLZ"), max_length=255, blank=True)
    place = models.CharField(verbose_name=_("Place"), max_length=255, blank=True)

    payment_option = models.ForeignKey(
        PaymentOption,
        models.SET_NULL,
        null=True,
        blank=True,
        verbose_name=_("Selected payment option"),
    )

    digital_products_key = models.TextField(verbose_name=_("Digital products key"), blank=True)

    def get_confirm_url(self, request):
        return request.build_absolute_uri(reverse("confirm_order", args=[self.confirm_key]))

    def get_digital_products_url(self, request):
        return request.build_absolute_uri(
            reverse("digital_products", args=[self.digital_products_key])
        )

    @property
    def email_recipients(self):
        return [f'"{self.full_name}" <{self.email}>']

    def submit(self, request):
        self.submitted = True
        self.save()
        self.send_overview(request)

    def send_overview(self, request):
        send_templated_mail(
            template_name="overview",
            from_email=self.form.email_sender,
            recipient_list=self.email_recipients,
            context={"order": self, "confirm_url": self.get_confirm_url(request)},
        )

    def confirm(self, request):
        self.confirmed = True
        self.save()
        send_templated_mail(
            template_name="confirmation",
            from_email=self.form.email_sender,
            recipient_list=self.email_recipients,
            context={"order": self, "confirm_url": self.get_confirm_url(request)},
        )

    def send_pay_confirmation(self):
        self.paid = True
        self.save()
        send_templated_mail(
            template_name="pay_confirmation",
            from_email=self.form.email_sender,
            recipient_list=self.email_recipients,
            context={"order": self},
        )

    def send_confirmation_reminder(self, request):
        send_templated_mail(
            template_name="confirmation_reminder",
            from_email=self.form.email_sender,
            recipient_list=self.email_recipients,
            context={"order": self, "confirm_url": self.get_confirm_url(request)},
        )

    def send_pay_reminder(self, request):
        send_templated_mail(
            template_name="pay_reminder",
            from_email=self.form.email_sender,
            recipient_list=self.email_recipients,
            context={"order": self},
        )

    def send_digital_products_link(self, request):
        send_templated_mail(
            template_name="digital_products",
            from_email=self.form.email_sender,
            recipient_list=self.email_recipients,
            context={"order": self, "digital_products_url": self.get_digital_products_url(request)},
        )

    def __str__(self):
        return f"{self.form}: {self.full_name} [{self.created}]"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if not self.confirm_key:
            self.confirm_key = uuid4()
        if not self.digital_products_key:
            self.digital_products_key = uuid4()

    @property
    def items_count(self):
        return self.items.aggregate(models.Sum("count")).get("count__sum") or 0

    @property
    def items_count_shipping(self):
        return (
            self.items.aggregate(
                count__sum=models.Sum("count", filter=Q(item__count_for_shipping=True))
            ).get("count__sum")
            or 0
        )

    @property
    def shipping_price(self):
        return (
            self.shipping_option.get_price(self.items_count_shipping) if self.shipping_option else 0
        )

    @property
    def payment_price(self):
        return self.payment_option.price if self.payment_option else 0

    @property
    def total(self):
        total = 0
        for item in self.items.all():
            total += item.total
        total += self.shipping_price
        total += self.payment_price
        return total

    class Meta:
        verbose_name = _("Order")
        verbose_name_plural = _("Orders")
        permissions = [("manage_orders", _("Can manage orders"))]


class OrderItem(ExtensibleModel):
    count = models.IntegerField(verbose_name=_("Count"))
    item = models.ForeignKey(to=Item, on_delete=models.CASCADE, verbose_name=_("Item"))
    order = models.ForeignKey(
        to=Order, on_delete=models.CASCADE, verbose_name=_("Order"), related_name="items"
    )

    def create_share(self):
        if self.item.digital_product_object:
            return self.item.digital_product_object.create_share(self)

    @property
    def current_shares(self):
        if self.item.digital_product_object:
            return self.shares.filter(share_expiration__gte=timezone.now())
        return self.shares.none()

    @property
    def shares_left(self) -> int:
        shares_left = self.count - self.current_shares.count()
        return shares_left if shares_left > 0 else 0

    @property
    def total(self):
        return self.count * self.item.price



from django.urls import path
from django.views.generic import TemplateView

from . import views

urlpatterns = [
    path("list/", views.OrderListView.as_view(), name="list_orders"),
    path("list/<int:pk>/", views.OrderDetailView.as_view(), name="show_order"),
    path("list/<int:pk>/edit/", views.OrderEditView.as_view(), name="edit_order"),
    path("list/<int:pk>/delete/", views.OrderDeleteView.as_view(), name="delete_order"),
    path("confirm/<str:key>/", views.OrderConfirmView.as_view(), name="confirm_order"),
    path(
        "digital_products/",
        TemplateView.as_view(template_name="core/empty.html"),
        name="digital_products",
    ),
    path(
        "digital_products/<str:key>/",
        TemplateView.as_view(template_name="core/empty.html"),
        name="digital_products",
    ),
    path("pick_up/", views.OrderPickUpFormView.as_view(), name="pick_up_order"),
]



# Generated by Django 5.0.8 on 2024-08-24 14:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0019_auto_20230309_2157'),
    ]

    operations = [
        migrations.AddField(
            model_name='digitalproduct',
            name='managed_by_app_label',
            field=models.CharField(blank=True, editable=False, max_length=255, verbose_name='App label of app responsible for managing this instance'),
        ),
        migrations.AddField(
            model_name='digitalproductshare',
            name='managed_by_app_label',
            field=models.CharField(blank=True, editable=False, max_length=255, verbose_name='App label of app responsible for managing this instance'),
        ),
        migrations.AddField(
            model_name='item',
            name='managed_by_app_label',
            field=models.CharField(blank=True, editable=False, max_length=255, verbose_name='App label of app responsible for managing this instance'),
        ),
        migrations.AddField(
            model_name='nextcloudprovider',
            name='managed_by_app_label',
            field=models.CharField(blank=True, editable=False, max_length=255, verbose_name='App label of app responsible for managing this instance'),
        ),
        migrations.AddField(
            model_name='order',
            name='managed_by_app_label',
            field=models.CharField(blank=True, editable=False, max_length=255, verbose_name='App label of app responsible for managing this instance'),
        ),
        migrations.AddField(
            model_name='orderform',
            name='managed_by_app_label',
            field=models.CharField(blank=True, editable=False, max_length=255, verbose_name='App label of app responsible for managing this instance'),
        ),
        migrations.AddField(
            model_name='orderitem',
            name='managed_by_app_label',
            field=models.CharField(blank=True, editable=False, max_length=255, verbose_name='App label of app responsible for managing this instance'),
        ),
        migrations.AddField(
            model_name='paymentoption',
            name='managed_by_app_label',
            field=models.CharField(blank=True, editable=False, max_length=255, verbose_name='App label of app responsible for managing this instance'),
        ),
        migrations.AddField(
            model_name='shippingoption',
            name='managed_by_app_label',
            field=models.CharField(blank=True, editable=False, max_length=255, verbose_name='App label of app responsible for managing this instance'),
        ),
        migrations.AddField(
            model_name='shippingoptionprice',
            name='managed_by_app_label',
            field=models.CharField(blank=True, editable=False, max_length=255, verbose_name='App label of app responsible for managing this instance'),
        ),
    ] + [
        migrations.RunSQL(
            f"ALTER TABLE order_{model_name} drop column if exists site_id;"
        ) for model_name  in
        [
            "nextcloudprovider",
            "digitalproductshare",
            "digitalproduct",
            "item",
            "paymentoption",
            "shippingoption",
            "shippingoptionprice",
            "orderform",
            "order",
            "orderitem",
        ]
    ]



# Generated by Django 3.2.16 on 2022-12-22 11:48

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0013_auto_20221214_2054'),
    ]

    operations = [
        migrations.AddField(
            model_name='order',
            name='person',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='core.person', verbose_name='Person'),
        ),
    ]



# Generated by Django 3.2.16 on 2022-12-31 17:41
import uuid

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0017_digitalproductshare'),
    ]

    operations = [
        migrations.AddField(
            model_name='order',
            name='digital_products_key',
            field=models.TextField(blank=True, verbose_name='Digital products key', default=uuid.uuid4),
        ),
        migrations.AlterField(
            model_name='digitalproductshare',
            name='order_item',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='shares', to='order.orderitem', verbose_name='Order item'),
        ),
    ]
